@@ -1,12 +1,21 @@
 import argparse
-import faulthandler
 import logging
-import signal
 import sys
 from pathlib import Path
 
 from libqtile.log_utils import get_default_log, init_log
-from libqtile.scripts import check, cmd_obj, migrate, run_cmd, shell, start, top, udev
+from libqtile.scripts import (
+    check,
+    cmd_obj,
+    launch,
+    migrate,
+    repl,
+    run_cmd,
+    shell,
+    start,
+    top,
+    udev,
+)
 
 try:
     # Python>3.7 can get the version from importlib
@@ -17,20 +26,7 @@ except PackageNotFoundError:
     VERSION = "dev"
 
 
-def check_folder(value):
-    path = Path(value)
-    if not path.parent.is_dir():
-        raise argparse.ArgumentTypeError("Log path destination folder does not exist.")
-    # init_log expects a Path object so return `path` not `value`
-    return path
-
-
 def main():
-    faulthandler.enable(all_threads=True)
-    # This is a bit unfortunate. We use SIGUSR1&2 for reloading config &
-    # restarting qtile, so we overload SIGWINCH here to dump threads.
-    faulthandler.register(signal.SIGWINCH, all_threads=True)
-
     parent_parser = argparse.ArgumentParser(add_help=False)
     parent_parser.add_argument(
         "-l",
@@ -46,7 +42,7 @@ def main():
         "--log-path",
         default=get_default_log(),
         dest="log_path",
-        type=check_folder,
+        type=Path,
         help="Set alternative qtile log path",
     )
     main_parser = argparse.ArgumentParser(
@@ -69,12 +65,16 @@ def main():
     check.add_subcommand(subparsers, [parent_parser])
     migrate.add_subcommand(subparsers, [parent_parser])
     udev.add_subcommand(subparsers, [parent_parser])
+    launch.add_subcommand(subparsers, [parent_parser])
+    repl.add_subcommand(subparsers, [parent_parser])
 
     # `qtile help` should print help
     def print_help(options):
         main_parser.print_help()
 
-    help_ = subparsers.add_parser("help", help="Print help message and exit.")
+    help_ = subparsers.add_parser(
+        "help", help="Print help message and exit.", parents=[parent_parser]
+    )
     help_.set_defaults(func=print_help)
 
     options = main_parser.parse_args()
