@@ -18,7 +18,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 """
-    A command shell for Qtile.
+A command shell for Qtile.
 """
 
 from __future__ import annotations
@@ -45,12 +45,37 @@ if TYPE_CHECKING:
     from typing import Any
 
 
+# From: https://stackoverflow.com/a/64333329/3087339
+# Matches commas that are not enclosed in quote marks
+COMMA_MATCHER = re.compile(r",(?=(?:[^\"']*[\"'][^\"']*[\"'])*[^\"']*$)")
+
+# Matches string in the form of "key=value" and stores results in
+# groups named "key" and "value"
+KWARG_MATCHER = re.compile(r"(?P<key>\w+)\s?=\s*(?P<value>.*)")
+
+
+def split_args(args):
+    pos_args = []
+    kwargs = {}
+    for arg in args:
+        if kwarg := KWARG_MATCHER.match(arg.strip()):
+            kwargs[kwarg.group("key")] = tidy_str(kwarg.group("value"))
+        else:
+            pos_args.append(tidy_str(arg))
+
+    return pos_args, kwargs
+
+
+def tidy_str(text):
+    return text.strip(""" "'\t\r\n""")
+
+
 def terminal_width():
     width = None
     try:
         cr = struct.unpack("hh", fcntl.ioctl(0, termios.TIOCGWINSZ, "1234"))
         width = int(cr[1])
-    except (IOError, ImportError):
+    except (OSError, ImportError):
         pass
     return width or 80
 
@@ -79,7 +104,7 @@ class QSh:
             options = self._builtins + self._command_client.commands
             lst = [i for i in options if i.startswith(arg)]
             return lst
-        elif buf.startswith("cd ") or buf.startswith("ls "):
+        elif buf.startswith(("cd ", "ls ")):
             path, sep, last = arg.rpartition("/")
             node, rest_path = self._find_path(path)
 
@@ -99,7 +124,7 @@ class QSh:
 
     @property
     def prompt(self) -> str:
-        return "{} > ".format(format_selectors(self._command_client.selectors))
+        return f"{format_selectors(self._command_client.selectors)} > "
 
     def columnize(self, lst, update_termwidth=True) -> str:
         if update_termwidth:
@@ -207,7 +232,7 @@ class QSh:
         else:
             allow_root, _ = next_node.items(rest_path)
             if not allow_root:
-                return "Item required for {}".format(rest_path)
+                return f"Item required for {rest_path}"
             self._command_client = next_node.navigate(rest_path, None)
 
         return format_selectors(self._command_client.selectors) or "/"
@@ -235,8 +260,8 @@ class QSh:
 
         objects, items = self._ls(node, rest_path)
 
-        formatted_ls = ["{}{}/".format(base_path, i) for i in objects] + [
-            "{}[{}]/".format(base_path[:-1], i) for i in items
+        formatted_ls = [f"{base_path}{i}/" for i in objects] + [
+            f"{base_path[:-1]}[{i}]/" for i in items
         ]
         return self.columnize(formatted_ls)
 
@@ -298,7 +323,7 @@ class QSh:
             assert ret is not None
             return ret
         else:
-            return "No such command: %s" % arg
+            return f"No such command: {arg}"
 
     def do_exit(self, args) -> None:
         """Exit qshell"""
@@ -317,30 +342,35 @@ class QSh:
                 val = builtin(args)
                 return val
             else:
-                return "Invalid builtin: {}".format(cmd)
+                return f"Invalid builtin: {cmd}"
 
         command_match = re.fullmatch(r"(?P<cmd>\w+)\((?P<args>.*)\)", line)
         if command_match:
             cmd = command_match.group("cmd")
             args = command_match.group("args")
+            cmd_args: tuple
+            kwargs: dict[str, str] = {}
             if args:
-                cmd_args = tuple(map(str.strip, args.split(",")))
+                if cmd == "eval":
+                    # For eval, the whole argument should be a single string of code
+                    # so we shouldn't split it.
+                    cmd_args = (args,)
+                else:
+                    # For everything else, we split by commas except where they
+                    # are enclosed in quotation marks (i.e. part of a string)
+                    cmd_args, kwargs = split_args(COMMA_MATCHER.split(args))
             else:
                 cmd_args = ()
 
             if cmd not in self._command_client.commands:
-                return "Command does not exist: {}".format(cmd)
+                return f"Command does not exist: {cmd}"
 
             try:
-                return self._command_client.call(cmd, *cmd_args)
+                return self._command_client.call(cmd, *cmd_args, **kwargs)
             except CommandException as e:
-                return (
-                    "Caught command exception (is the command invoked incorrectly?): {}\n".format(
-                        e
-                    )
-                )
+                return f"Caught command exception (is the command invoked incorrectly?): {e}\n"
 
-        return "Invalid command: {}".format(line)
+        return f"Invalid command: {line}"
 
     def loop(self) -> None:
         self.readline.set_completer(self.complete)
@@ -359,7 +389,7 @@ class QSh:
             try:
                 val = self.process_line(line)
             except CommandError as e:
-                val = "Caught command error (is the current path still valid?): {}\n".format(e)
+                val = f"Caught command error (is the current path still valid?): {e}\n"
             if isinstance(val, str):
                 print(val)
             elif val:
