@@ -407,7 +407,7 @@ class Battery(base.ThreadPoolText):
 
     .. code-block:: python
 
-        Battery(..., charge_controller: lambda (0, 90))
+        Battery(..., charge_controller=lambda: (0, 90))
 
     The battery widget also supplies some charging algorithms. To only charge
     the battery between 40-50% while connected to a thunderbolt docking
@@ -416,7 +416,7 @@ class Battery(base.ThreadPoolText):
     .. code-block:: python
 
         from libqtile.widget.battery import thunderbolt_smart_charge
-        Battery(..., charge_controller: thunderbolt_smart_charge)
+        Battery(..., charge_controller=thunderbolt_smart_charge)
 
     To temporarily disable/re-enable this (e.g. if you know you're
     going mobile and need to charge) use either:
@@ -450,10 +450,34 @@ class Battery(base.ThreadPoolText):
         ("unknown_char", "?", "Character to indicate the battery status is unknown"),
         ("format", "{char} {percent:2.0%} {hour:d}:{min:02d} {watt:.2f} W", "Display format"),
         ("hide_threshold", None, "Hide the text when there is enough energy 0 <= x < 1"),
-        ("show_short_text", True, 'Show "Full" or "Empty" rather than formated text'),
+        (
+            "full_short_text",
+            "Full",
+            "Short text to indicate battery is full; see `show_short_text`",
+        ),
+        (
+            "empty_short_text",
+            "Empty",
+            "Short text to indicate battery is empty; see `show_short_text`",
+        ),
+        (
+            "show_short_text",
+            True,
+            "Show only characters rather than formatted text when battery is full or empty",
+        ),
         ("low_percentage", 0.10, "Indicates when to use the low_foreground color 0 < x < 1"),
         ("low_foreground", "FF0000", "Font color on low battery"),
         ("low_background", None, "Background color on low battery"),
+        (
+            "charging_foreground",
+            None,
+            "Font color on charging battery. Set to None to disable.",
+        ),
+        (
+            "charging_background",
+            None,
+            "Background color on charging battery. Set to None to disable.",
+        ),
         ("update_interval", 60, "Seconds between status updates"),
         ("battery", 0, "Which battery should be monitored (battery number or name)"),
         ("notify_below", None, "Send a notification below this battery level."),
@@ -540,6 +564,9 @@ class Battery(base.ThreadPoolText):
             if status.state == BatteryState.DISCHARGING and status.percent < self.low_percentage:
                 self.layout.colour = self.low_foreground
                 self.background = self.low_background
+            elif status.state == BatteryState.CHARGING:
+                self.layout.colour = self.charging_foreground or self.foreground
+                self.background = self.charging_background or self.normal_background
             else:
                 self.layout.colour = self.foreground
                 self.background = self.normal_background
@@ -550,13 +577,13 @@ class Battery(base.ThreadPoolText):
             char = self.discharge_char
         elif status.state == BatteryState.FULL:
             if self.show_short_text:
-                return "Full"
+                return self.full_short_text
             char = self.full_char
         elif status.state == BatteryState.EMPTY or (
             status.state == BatteryState.UNKNOWN and status.percent == 0
         ):
             if self.show_short_text:
-                return "Empty"
+                return self.empty_short_text
             char = self.empty_char
         elif status.state == BatteryState.NOT_CHARGING:
             char = self.not_charging_char
@@ -585,7 +612,7 @@ class BatteryIcon(base._Widget):
         ("battery", 0, "Which battery should be monitored"),
         ("update_interval", 60, "Seconds between status updates"),
         ("theme_path", default_icon_path(), "Path of the icons"),
-        ("scale", 1, "Scale factor relative to the bar height.  " "Defaults to 1"),
+        ("scale", 1, "Scale factor relative to the bar height.  Defaults to 1"),
         ("padding", 0, "Additional padding either side of the icon"),
     ]
 
@@ -660,7 +687,7 @@ class BatteryIcon(base._Widget):
         self.drawer.ctx.set_source(image.pattern)
         self.drawer.ctx.paint()
         self.drawer.ctx.restore()
-        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.length)
+        self.draw_at_default_position()
 
     @staticmethod
     def _get_icon_key(status: BatteryStatus) -> str:
