@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2011 Florian Mounier
 # Copyright (c) 2011 Kenji_Takahashi
 # Copyright (c) 2012 roger
@@ -36,46 +35,8 @@ from libqtile.widget import base
 
 class CurrentLayout(base._TextBox):
     """
-    Display the name of the current layout of the current group of the screen,
-    the bar containing the widget, is on.
-    """
-
-    def __init__(self, width=bar.CALCULATED, **config):
-        base._TextBox.__init__(self, "", width, **config)
-
-    def _configure(self, qtile, bar):
-        base._TextBox._configure(self, qtile, bar)
-        layout_id = self.bar.screen.group.current_layout
-        self.text = self.bar.screen.group.layouts[layout_id].name
-        self.setup_hooks()
-
-        self.add_callbacks(
-            {
-                "Button1": qtile.next_layout,
-                "Button2": qtile.prev_layout,
-            }
-        )
-
-    def hook_response(self, layout, group):
-        if group.screen is not None and group.screen == self.bar.screen:
-            self.text = layout.name
-            self.bar.draw()
-
-    def setup_hooks(self):
-        hook.subscribe.layout_change(self.hook_response)
-
-    def remove_hooks(self):
-        hook.unsubscribe.layout_change(self.hook_response)
-
-    def finalize(self):
-        self.remove_hooks()
-        base._TextBox.finalize(self)
-
-
-class CurrentLayoutIcon(base._TextBox):
-    """
-    Display the icon representing the current layout of the
-    current group of the screen on which the bar containing the widget is.
+    Display the icon or the name of the current layout of the current group
+    of the screen on which the bar containing the widget is.
 
     If you are using custom layouts, a default icon with question mark
     will be displayed for them. If you want to use custom icon for your own
@@ -93,86 +54,158 @@ class CurrentLayoutIcon(base._TextBox):
     - built-in qtile icons
     """
 
-    orientations = base.ORIENTATION_HORIZONTAL
-
     defaults = [
-        ("scale", 1, "Scale factor relative to the bar height. Defaults to 1"),
+        ("icon_first", True, "When ``mode='both'``, the icon will display before the text."),
+        (
+            "mode",
+            "text",
+            "Display only the name of the layout with ``text`` "
+            "or only the icon of the layout with ``icon``. "
+            "Altenatively display the two in order with ``both``",
+        ),
+        ("scale", 1, "Icon's scale factor relative to the bar height."),
         (
             "custom_icon_paths",
             [],
-            "List of folders where to search icons before"
-            "using built-in icons or icons in ~/.icons dir.  "
-            "This can also be used to provide"
-            "missing icons for custom layouts.  "
-            "Defaults to empty list.",
+            "List of folders where to search icons before "
+            "using built-in icons or icons in ~/.icons dir. "
+            "This can also be used to provide "
+            "missing icons for custom layouts.",
         ),
     ]
 
-    def __init__(self, **config):
-        base._TextBox.__init__(self, "", **config)
-        self.add_defaults(CurrentLayoutIcon.defaults)
+    def __init__(self, width=bar.CALCULATED, **config):
+        base._TextBox.__init__(self, "", width, **config)
+        self.add_defaults(CurrentLayout.defaults)
 
-        self.length_type = bar.STATIC
-        self.length = 0
+        self.icons_loaded = False
+        self.img_length = 0
 
     def _configure(self, qtile, bar):
+        assert self.mode in ("text", "icon", "both")
         base._TextBox._configure(self, qtile, bar)
         layout_id = self.bar.screen.group.current_layout
         self.text = self.bar.screen.group.layouts[layout_id].name
-        self.current_layout = self.text
-        self.icons_loaded = False
         self.icon_paths = []
         self.surfaces = {}
         self._update_icon_paths()
         self._setup_images()
-        self._setup_hooks()
+        self.setup_hooks()
 
         self.add_callbacks(
             {
                 "Button1": qtile.next_layout,
                 "Button2": qtile.prev_layout,
+                "Button3": self.change_draw_method,
             }
         )
 
+    @property
+    def current_layout(self):
+        return self.text
+
+    def calculate_length(self):
+        if self.mode == "text":
+            return base._TextBox.calculate_length(self)
+        elif self.mode == "icon":
+            return self.img_length + self.actual_padding * 2
+        # self.mode == "both"
+        # add only one padding because base._TextBox.calculate_length already add two
+        return base._TextBox.calculate_length(self) + self.img_length + self.actual_padding
+
     def hook_response(self, layout, group):
         if group.screen is not None and group.screen == self.bar.screen:
-            self.current_layout = layout.name
+            self.text = layout.name
             self.bar.draw()
 
-    def _setup_hooks(self):
+    def setup_hooks(self):
         """
         Listens for layout change and performs a redraw when it occurs.
         """
         hook.subscribe.layout_change(self.hook_response)
 
-    def _remove_hooks(self):
+    def remove_hooks(self):
         """
         Listens for layout change and performs a redraw when it occurs.
         """
         hook.unsubscribe.layout_change(self.hook_response)
 
+    def change_draw_method(self):
+        if self.mode == "both":
+            return
+        self.mode = "text" if self.mode != "text" else "icon"
+        self.bar.draw()
+
     def draw(self):
-        if self.icons_loaded:
-            try:
-                surface = self.surfaces[self.current_layout]
-            except KeyError:
-                logger.error("No icon for layout %s", self.current_layout)
-            else:
-                self.drawer.clear(self.background or self.bar.background)
-                self.drawer.ctx.save()
-                self.drawer.ctx.translate(
-                    (self.width - surface.width) / 2,
-                    (self.bar.height - surface.height) / 2,
-                )
-                self.drawer.ctx.set_source(surface.pattern)
-                self.drawer.ctx.paint()
-                self.drawer.ctx.restore()
-                self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.length)
+        if self.mode != "text":
+            self.draw_icon()
         else:
-            # Fallback to text
-            self.text = self.current_layout[0].upper()
             base._TextBox.draw(self)
 
+    def draw_icon(self):
+        if not self.icons_loaded:
+            return
+        try:
+            surface = self.surfaces[self.current_layout]
+        except KeyError:
+            logger.error("No icon for layout %s", self.current_layout)
+            return
+
+        self.drawer.clear(self.background or self.bar.background)
+        self.drawer.ctx.save()
+        self.rotate_drawer()
+
+        translatex, translatey = self.width, self.height
+
+        if self.mode == "both":
+            if self.bar.horizontal:
+                height = self.bar.height
+                if self.icon_first:
+                    # padding - icon - padding - text - padding
+                    x = self.actual_padding + self.img_length + self.actual_padding
+                    translatex -= base._TextBox.calculate_length(self) - self.actual_padding
+                else:
+                    # padding - text - padding - icon - padding
+                    x = self.actual_padding
+                    translatex += base._TextBox.calculate_length(self) - self.actual_padding
+            elif self.rotate:
+                height = self.bar.width
+                if self.icon_first:
+                    # padding - icon - padding - text - padding
+                    x = self.actual_padding + self.img_length + self.actual_padding
+                    translatey -= base._TextBox.calculate_length(self) - self.actual_padding
+                else:
+                    # padding - text - padding - icon - padding
+                    x = self.actual_padding
+                    translatey += base._TextBox.calculate_length(self) - self.actual_padding
+            else:
+                x = 0
+                if self.icon_first:
+                    # padding - icon - padding - text - padding
+                    height = self.actual_padding + self.img_length + self.actual_padding
+                    translatey -= base._TextBox.calculate_length(self) - self.actual_padding
+                else:
+                    # padding - text - padding - icon - padding
+                    height = self.actual_padding
+                    translatey += base._TextBox.calculate_length(self) - self.actual_padding
+                # neutralize all math in the layout.draw() below
+                # to simulate starting height from zero
+                height = (height * 2) + self.layout.height - 2
+
+            self.layout.draw(x, int(height / 2 - self.layout.height / 2) + 1)
+
+        if not self.bar.horizontal and self.rotate:
+            translatex, translatey = translatey, translatex
+
+        self.drawer.ctx.translate(
+            (translatex - surface.width) / 2,
+            (translatey - surface.height) / 2,
+        )
+        self.drawer.ctx.set_source(surface.pattern)
+        self.drawer.ctx.paint()
+        self.drawer.ctx.restore()
+        self.draw_at_default_position()
+
     def _get_layout_names(self):
         """
         Returns a sequence of tuples of layout name and lowercased class name
@@ -205,7 +238,7 @@ class CurrentLayoutIcon(base._TextBox):
     def find_icon_file_path(self, layout_name):
         for icon_path in self.icon_paths:
             for extension in ["png", "svg"]:
-                icon_filename = "layout-{}.{}".format(layout_name, extension)
+                icon_filename = f"layout-{layout_name}.{extension}"
                 icon_file_path = os.path.join(icon_path, icon_filename)
                 if os.path.isfile(icon_file_path):
                     return icon_file_path
@@ -214,6 +247,9 @@ class CurrentLayoutIcon(base._TextBox):
         """
         Loads layout icons.
         """
+        width = (self.bar.width - 2) * self.scale if not self.bar.horizontal else None
+        height = (self.bar.height - 2) * self.scale if self.bar.horizontal else None
+
         for names in self._get_layout_names():
             layout_name = names[0]
             # Python doesn't have an ordered set but we can use a dictionary instead
@@ -231,15 +267,14 @@ class CurrentLayoutIcon(base._TextBox):
 
             img = Img.from_path(icon_file_path)
 
-            new_height = (self.bar.height - 2) * self.scale
-            img.resize(height=new_height)
-            if img.width > self.length:
-                self.length = img.width + self.actual_padding * 2
+            img.resize(width=width, height=height)
+            if img.width > self.img_length:
+                self.img_length = img.width
 
             self.surfaces[layout_name] = img
 
         self.icons_loaded = True
 
     def finalize(self):
-        self._remove_hooks()
+        self.remove_hooks()
         base._TextBox.finalize(self)
