@@ -32,6 +32,21 @@ class Net(base.ThreadPoolText):
     """
     Displays interface down and up speed
 
+    The following fields are available in the `format` string:
+
+    - ``interface``: name of the interface
+    - ``down``: download speed
+    - ``down_suffix``: suffix for the download speed
+    - ``down_cumulative``: cumulative download traffic
+    - ``down_cumulative_suffix``: suffix for the cumulative download traffic
+    - ``up``: upload speed
+    - ``up_suffix``: suffix for the upload speed
+    - ``up_cumulative``: cumulative upload traffic
+    - ``up_cumulative_suffix``: suffix for the cumulative upload traffic
+    - ``total``: total speed
+    - ``total_suffix``: suffix for the total speed
+    - ``total_cumulative``: cumulative total traffic
+    - ``total_cumulative_suffix``: suffix for the cumulative total traffic
 
     Widget requirements: psutil_.
 
@@ -41,7 +56,7 @@ class Net(base.ThreadPoolText):
     defaults = [
         (
             "format",
-            "{interface}: {down} \u2193\u2191 {up}",
+            "{interface}: {down:6.2f}{down_suffix:<2}\u2193\u2191{up:6.2f}{up_suffix:<2}",
             "Display format of down/upload/total speed of given interfaces",
         ),
         (
@@ -53,6 +68,11 @@ class Net(base.ThreadPoolText):
         ("update_interval", 1, "The update interval."),
         ("use_bits", False, "Use bits instead of bytes per second?"),
         ("prefix", None, "Use a specific prefix for the unit of the speed."),
+        (
+            "cumulative_prefix",
+            None,
+            "Use a specific prefix for the unit of the cumulative traffic.",
+        ),
     ]
 
     def __init__(self, **config):
@@ -78,23 +98,23 @@ class Net(base.ThreadPoolText):
                 self.interface = [self.interface]
             else:
                 raise AttributeError(
-                    "Invalid Argument passed: %s\nAllowed Types: list, str, None" % self.interface
+                    f"Invalid Argument passed: {self.interface}\nAllowed Types: list, str, None"
                 )
         self.stats = self.get_stats()
 
-    def convert_b(self, num_bytes: float) -> tuple[float, str]:
+    def convert_b(self, num_bytes: float, prefix: str | None = None) -> tuple[float, str]:
         """Converts the number of bytes to the correct unit"""
 
         num_bytes *= self.byte_multiplier
 
-        if self.prefix is None:
+        if prefix is None:
             if num_bytes > 0:
                 power = int(log(num_bytes) / log(self.factor))
                 power = min(power, len(self.units) - 1)
             else:
                 power = 0
         else:
-            power = self.allowed_prefixes.index(self.prefix)
+            power = self.allowed_prefixes.index(prefix)
 
         converted_bytes = num_bytes / self.factor**power
         unit = self.units[power]
@@ -123,15 +143,6 @@ class Net(base.ThreadPoolText):
                 }
             return interfaces
 
-    def _format(self, down, down_letter, up, up_letter, total, total_letter):
-        max_len_down = 7 - len(down_letter)
-        max_len_up = 7 - len(up_letter)
-        max_len_total = 7 - len(total_letter)
-        down = "{val:{max_len}.2f}".format(val=down, max_len=max_len_down)
-        up = "{val:{max_len}.2f}".format(val=up, max_len=max_len_up)
-        total = "{val:{max_len}.2f}".format(val=total, max_len=max_len_total)
-        return down[:max_len_down], up[:max_len_up], total[:max_len_total]
-
     def poll(self):
         ret_stat = []
         try:
@@ -144,24 +155,37 @@ class Net(base.ThreadPoolText):
                 down = down / self.update_interval
                 up = up / self.update_interval
                 total = total / self.update_interval
-                down, down_letter = self.convert_b(down)
-                up, up_letter = self.convert_b(up)
-                total, total_letter = self.convert_b(total)
-                down, up, total = self._format(
-                    down, down_letter, up, up_letter, total, total_letter
+                down, down_suffix = self.convert_b(down, self.prefix)
+                down_cumulative, down_cumulative_suffix = self.convert_b(
+                    new_stats[intf]["down"], self.cumulative_prefix
+                )
+                up, up_suffix = self.convert_b(up, self.prefix)
+                up_cumulative, up_cumulative_suffix = self.convert_b(
+                    new_stats[intf]["up"], self.cumulative_prefix
+                )
+                total, total_suffix = self.convert_b(total, self.prefix)
+                total_cumulative, total_cumulative_suffix = self.convert_b(
+                    new_stats[intf]["total"], self.cumulative_prefix
                 )
                 self.stats[intf] = new_stats[intf]
                 ret_stat.append(
                     self.format.format(
-                        **{
-                            "interface": intf,
-                            "down": down + down_letter,
-                            "up": up + up_letter,
-                            "total": total + total_letter,
-                        }
+                        interface=intf,
+                        down=down,
+                        down_suffix=down_suffix,
+                        down_cumulative=down_cumulative,
+                        down_cumulative_suffix=down_cumulative_suffix,
+                        up=up,
+                        up_suffix=up_suffix,
+                        up_cumulative=up_cumulative,
+                        up_cumulative_suffix=up_cumulative_suffix,
+                        total=total,
+                        total_suffix=total_suffix,
+                        total_cumulative=total_cumulative,
+                        total_cumulative_suffix=total_cumulative_suffix,
                     )
                 )
 
             return " ".join(ret_stat)
-        except Exception as excp:
-            logger.error("%s: Caught Exception:\n%s", self.__class__.__name__, excp)
+        except Exception:
+            logger.exception("Net widget errored while polling:")
